Package: DisImpact
Title: Calculates Disproportionate Impact When Binary Success Data are
        Disaggregated by Subgroups
Version: 0.0.21
Authors@R: person("Vinh", "Nguyen", email = "nguyenvq714@gmail.com", role = c("aut", "cre"))
Description: Implements methods for calculating disproportionate impact: the percentage point gap, proportionality index, and the 80% index.
 California Community Colleges Chancellor's Office (2017).  Percentage Point Gap Method. <https://www.cccco.edu/-/media/CCCCO-Website/About-Us/Divisions/Digital-Innovation-and-Infrastructure/Research/Files/PercentagePointGapMethod2017.ashx>.
 California Community Colleges Chancellor's Office (2014).  Guidelines for Measuring Disproportionate Impact in Equity Plans. <https://www.cccco.edu/-/media/CCCCO-Website/Files/DII/guidelines-for-measuring-disproportionate-impact-in-equity-plans-tfa-ada.pdf>.
Depends: R (>= 3.4.0)
Imports: dplyr (>= 0.8.5), rlang, tidyselect, purrr, tidyr, parallel,
        fst, DBI, duckdb (>= 0.5.0), glue, stringr, data.table (>=
        1.14.2), collapse, sets
License: GPL-3
URL: https://github.com/vinhdizzo/DisImpact
BugReports: https://github.com/vinhdizzo/DisImpact/issues
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.1.2
Suggests: knitr, rmarkdown, markdown, prettydoc, ggplot2, forcats,
        scales, tinytest
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2022-10-10 17:26:29 UTC; vnguyen216
Author: Vinh Nguyen [aut, cre]
Maintainer: Vinh Nguyen <nguyenvq714@gmail.com>
Repository: CRAN
Date/Publication: 2022-10-10 18:00:02 UTC
Built: R 4.6.0; ; 2026-04-29 15:44:33 UTC; unix
